"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const assert = require("assert");
const crypto = require("crypto");
const qs = require("query-string");
const got = require("got");
const json2typescript_1 = require("json2typescript");
const https_1 = require("https");
let BittrexResponse = class BittrexResponse {
    constructor() {
        this.success = undefined;
        this.message = undefined;
        this.result = undefined;
    }
};
tslib_1.__decorate([
    json2typescript_1.JsonProperty(),
    tslib_1.__metadata("design:type", Boolean)
], BittrexResponse.prototype, "success", void 0);
tslib_1.__decorate([
    json2typescript_1.JsonProperty(),
    tslib_1.__metadata("design:type", String)
], BittrexResponse.prototype, "message", void 0);
tslib_1.__decorate([
    json2typescript_1.JsonProperty(),
    tslib_1.__metadata("design:type", Object)
], BittrexResponse.prototype, "result", void 0);
BittrexResponse = tslib_1.__decorate([
    json2typescript_1.JsonObject
], BittrexResponse);
exports.BittrexResponse = BittrexResponse;
class Transport {
    constructor({ key, secret, version = '1.1', agent = false }) {
        this.jsonConvert = new json2typescript_1.JsonConvert();
        assert(!!key, 'key is required');
        assert(!!secret, 'secret is required');
        this.transportOptions = {
            key: key,
            secret: secret,
            version: version,
            agent: agent
        };
        if (agent && typeof agent === 'boolean') {
            this.transportOptions.agent = new https_1.Agent({
                keepAlive: true,
                maxSockets: 100,
                maxFreeSockets: 10
            });
        }
    }
    get baseUrl() {
        return `https://bittrex.com/api/v${this.transportOptions.version}`;
    }
    async request(responseType, pathname, data = {}) {
        const { url, opts } = this.prepareRequest(pathname, data);
        return got(url, opts).then((response) => {
            return this.handleResponse(responseType, response);
        });
    }
    handleResponse(responseType, response) {
        return new Promise((resolve, reject) => {
            let bittrexResponse = this.jsonConvert.deserialize(response.body, BittrexResponse);
            if (bittrexResponse.success) {
                let converted;
                if (bittrexResponse.result instanceof Array) {
                    converted = this.jsonConvert.deserializeArray(bittrexResponse.result, responseType);
                }
                else {
                    converted = this.jsonConvert.deserialize(bittrexResponse.result, responseType);
                }
                return resolve(converted);
            }
            else {
                return reject(bittrexResponse);
            }
        });
    }
    prepareRequest(pathname, data = {}) {
        const url = `${this.baseUrl}${pathname}`;
        const query = Object.assign({ apikey: this.transportOptions.key, nonce: Math.floor(Date.now() / 1000) }, this.removeUndefined(data));
        const urlSigned = `${url}?${qs.stringify(query)}`;
        const signature = crypto.createHmac('sha512', this.transportOptions.secret)
            .update(urlSigned, 'utf8')
            .digest('hex');
        const opts = {
            agent: this.transportOptions.agent,
            json: true,
            headers: {
                'User-Agent': 'Mozilla/4.0 (compatible; bittrex-typescript)',
                apisign: signature
            }
        };
        return { url: urlSigned, opts: opts };
    }
    removeUndefined(obj) {
        return Object.keys(obj).reduce((acc, key) => {
            const _acc = acc;
            if (obj[key] !== undefined)
                _acc[key] = obj[key];
            return _acc;
        }, {});
    }
}
exports.Transport = Transport;
//# sourceMappingURL=Transport.js.map